from wizard import *

if formencode_present:
    import formbuild

    def create_validation_state(flow, steps):
        validation_state = formbuild.ValidationState(flow)
        validation_state.validation_state.steps = steps
        return validation_state

    class RemoveLaterSteps(formencode.FancyValidator):
        """\
    This pre_validator needs to:
        * remove any fields associated with steps later than the current one and purge them from the wizard store
        * ensure that values for each of the steps prior to the current one are validated
        * The whole wizard is treated as a single namespace so there is no separation into steps. Which field is treated as being associated with which step is entirely up to the validators. As long as they validate it doesn't matter what extra fields are passed to them or from which step. This means when the wizard is being refactored, data from subsequent steps can still populate fields from earlier steps.
    
        """
        def to_python(self, values, state):
            steps = state.validation_state.steps
            final = {}
            need_purge = False
            for k, v in values.items():
                for step in steps:
                    if k.startswith(step+'.') or k == step:
                        final[k] = v
            return final
    
    class StripEmptySteps(formencode.FancyValidator):
        def to_python(self, values, state):
            # Remove empty steps
            for k, v in values.items():
                if isinstance(v, formencode.NoDefault):
                    del values[k]
            return values
    
    # There is a formencode limitation which means the current step needs passing via the state.
    
    import extrahelpers.validators
    
    class Wizard(formencode.Schema):
        allow_extra_fields = True
        filter_extra_fields = True
        pre_validators = [RemoveLaterSteps(), extrahelpers.validators.OneDepthNestedVariables()]
        # Would be good to add a merger one here too.
        chained_validators = [StripEmptySteps()]

